/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia 
 * Departamento de Sistemas e Computacao 
 * Autora: Elaine de M. Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para a Aquisicao de Dados
 * e o Controle em Tempo Real
 * ------------------------------------------------------------------------------------------
 *  Comedi network communication header.
 *
 *  Revision 1.1  2009/08/03 01:32:30  elaine
 *  Initial version
 *
 *  Revision 1.2  2009/12/05 10:22:50  elaine
 *  Minor enhancements
 *
 *  -----------------------------------------------------------------------------------------
 *   
 */   

#define SERVER_PORT 3490    // server binding port 
#define BUFFERSIZE 1024     // max buffer size 
#define BACKLOG 50          // queues up to 50 connections
#define MAX_PARAMS 10       // max number of parameters in a comedi function    

#ifndef _comedisrv_h_
#define _comedisrv_h_

/* Server functions */
extern void nullDev(char *sendStr, char *rcStr);
extern void clearStr(char *sendStr, char *rcStr);
extern void *getInAddr(struct sockaddr *sa);
extern int getLocalSocket(int port);
extern int getClientData(int sockfd, int newSockfd, char buffer[],int lim);
extern int parser(char *buffer, int lim, char **par);
extern void execComedi(int newSockfd, char **par, char *Msg, char *rc);
extern int sendMsg(char *statusMsg,const char *msg, int sock, int size);
extern int getDriverName(char *boardName, int lineSize, char *option, char *driver);
extern int execCmd(char *path, char **args);

/* Client functions */
extern int connectToServer(char *ip);
extern int clientSender(int sockfd, char *msg);
extern int clientReceiver(int sockfd, char *msg);
extern int checkCmdStatus(char *recvMsg, char *editedMsg);


/* Remote Comedi functions */
extern void remoteModprobe(char *acq,char *opt, char *modprobeMsg);
extern void remoteComediConfig(char *dev,char *acq,char *io,char *irq, char *opt, char *configMsg);
extern void remoteComediOpen(char *dev,char *openMsg);
extern void remoteComediGetMaxdata(char *dev, char *subdev, char *channel, char *maxDataMsg);
extern void remoteComediGetRange(char *dev, char *subdev, char *channel, char *range, char *rangeMsg);
extern void remoteComediToPhys(char *data, char *maxdata, char *toPhysMsg,char *min, char *max, char *unit);
extern void remoteComediFromPhys(char *data, char *maxdata, char *fromPhysMsg);
extern void remoteComediDataRead(char *dev, char *subdev, char *channel, char *range, char *aref,char *dataReadMsg);
extern void remoteComediDioRead(char *dev, char *subdev, char *channel, char *dioRMsg);
extern void remoteComediDioWrite(char *dev, char *subdev, char *channel, char *data, char *dioWMsg);
extern void remoteComediDataWrite(char *dev, char *subdev, char *channel, char *range, char *aref, char *data, char *dataWriteMsg);
extern void remoteGetVersion(char *dev, char *getVerMsg);
extern void remoteGetDriver(char *dev, char *getDriverMsg);
extern void remoteGetNSubdevices(char *dev, char *NSubdevMsg);
extern void remoteGetSubdeviceType(char *dev, char *subdev, char *getSubdevTypeMsg);
extern void remoteFindRange(char *dev, char *findRangeMsg);


/* Comedi to Scicos functions */
extern void verifySubdeviceType(char *typeStat, char *typeMsg,char *recvType, char *ip, char *dev, char *subdev, int *typesAllowed, int typesCount);
extern void openComediDevice(char *openStat, char *openMsg,char *recvOpen, char *ip, char *dev);
extern int digitalRead(char *dioRStat, char *dioRMsg,char *recvDioR, char *ip, char *dev, char *subdev, char *channel);
extern void getMaxData(char *maxDataStat, char *maxDataMsg,char *recvMaxData, char *ip, char *dev, char *subdev, char *channel);
extern void getRange(char *rangeStat, char *rangeMsg,char *recvRange, char *ip, char *dev, char *subdev, char *channel, char *range, char *minRange, char *maxRange, char *unitRange);
extern void analogRead(char *dataReadStat, char *dataReadMsg,char *recvDataRead, char *ip, char *dev, char *subdev, char *channel, char *range, char *aref);
extern double comediToPhys(char *recvMaxData, char *recvDataRead, char *toPhysStat, char *toPhysMsg, char *recvToPhys, char *min, char *max, char *unit, char *ip);
extern void comediFromPhys(double dataPhys, char *recvMaxData, char *fromPhysMsg, char * fromPhysStat, char *recvFromPhys, char *ip);
extern void analogWrite(char *dataWriteStat, char *dataWriteMsg,char *recvDataWrite, char * recvFromPhys, char *ip, char *dev, char *subdev, char *channel, char *range, char *aref);



//extern void itoa(int n, char s[]);

#endif /* _comedisrv_h_ */


