/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de Mattos Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para Aquisicao de Dados
 * e o Controle em Tempo Real
 * -----------------------------------------------------------------------------------------
 *  Comedi acquistion block: Computational function for the Comedi Digital Data Read block
 *
 *  Revision 1.1   2009/12/19  12:13:00 elaine
 *  Initial revision
 *
 *  Revision 1.2   2009/12/24  12:48:00 elaine
 *  Improved pointer allocation. Included error checking. 
 *
 *  Revision 1.3   2010/01/04  20:48:00 elaine
 *  Broke code in smaller functions (Comedi/Scilab functions).
 *
 *  ----------------------------------------------------------------------------------------
 */


/* required header files */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h> 
#include <fcntl.h> 
#include <ctype.h>
#include <sys/types.h> 
#include <asm/types.h> 
#include "../includes/scicos_block4.h"  /* Specific for scicos block developement */
#include "../includes/machine.h"
#include <math.h>
//#include <comedilib.h>     /* Specific for Comedi Interface */
#include <sys/socket.h>    /* Specific for socket connection */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "../includes/comedisrv.h"


/* simulation flag constants */
#define Init           4   /* Initialization */ 
#define OutputUpdate   1
#define StateUpdate    2
#define Integration    0
#define Mode           9   /* Zero Crossing */
#define EventScheduler 3
#define Ending         5   

typedef unsigned int lsampl_t;

/* simulation function for the Acquisition of digital signals */
void comedi_net_dg_in(scicos_block *block, int flag) {

	static int convflag = 1;
	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
	int execStatus;
	static char *openStat, *openMsg, *recvOpen;
	static char *dioRStat, *dioRMsg, *recvDioR;
	static char *typeStat, *typeMsg, *recvType;

	


	/* customer inputs from Scicos block */
	char ip[BUFFERSIZE];
	char dev[BUFFERSIZE];
	char subdev[BUFFERSIZE];
	char channel[BUFFERSIZE];
	int typesAllowed[2] = {3,5};
	int typesCount = 2;

	

	/* Scicos input and output vectors */
	int *ipar = block->ipar;
	double *outptr = block->outptr[0];

	/* customer input sizes */
	int devSize, ipSize,subdevSize,channSize;

	/* Scicos custom function to get strings from blocks to C programs */
	extern int cvstr_();


	

	switch(flag) {  

		case Init: // Initializes variables, starts memory allocation  


			/* initializes input parameter sizes coming from Scicos */ 
			ipSize = ipar[0];
			devSize = ipar[1];
			subdevSize = ipar[2];
			channSize = ipar[3];


                        /* initializes input parameters coming from Scicos */ 
			cvstr_(&ipSize,&ipar[4],ip,&convflag,ipSize*sizeof(char));
			ip[ipSize] = '\0';
			
			cvstr_(&devSize,&ipar[4+ipSize],dev,&convflag,devSize*sizeof(char));
			dev[devSize] = '\0';
	
			cvstr_(&subdevSize, &ipar[4+devSize+ipSize], subdev, &convflag, subdevSize*sizeof(char));
			subdev[subdevSize] = '\0';

			cvstr_(&channSize, &ipar[4+devSize+ipSize+subdevSize], channel, &convflag, channSize*sizeof(char));
			channel[channSize] = '\0';

			
			/* memory allocation to character pointers */
			
			/* comedi_open */
			if ((openMsg  = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((openStat = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvOpen = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			
			
			/* comedi_dio_read */
			if ((dioRStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((dioRMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvDioR = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			
			
			/* comedi_get_subdevice_type */
			if ((typeStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((typeMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvType = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}

			
			/* acquisition functions */
			openComediDevice(openStat, openMsg,recvOpen, ip, dev);
			verifySubdeviceType(typeStat, typeMsg, recvType, ip, dev, subdev,typesAllowed,typesCount);
			     
		      
		      
			break;
			
  case OutputUpdate:

		      
		      /* assignment from Scicos block inputs to program variables */
		     	
			ipSize = ipar[0];
			devSize = ipar[1];
			subdevSize = ipar[2];
			channSize = ipar[3];

			cvstr_(&ipSize,&ipar[4],ip,&convflag,ipSize*sizeof(char));
			ip[ipSize] = '\0';
			
			cvstr_(&devSize,&ipar[4+ipSize],dev,&convflag,devSize*sizeof(char));
			dev[devSize] = '\0';
	
			cvstr_(&subdevSize, &ipar[4+devSize+ipSize], subdev, &convflag, subdevSize*sizeof(char));
			subdev[subdevSize] = '\0';

			cvstr_(&channSize, &ipar[4+devSize+ipSize+subdevSize], channel, &convflag, channSize*sizeof(char));
			channel[channSize] = '\0';


			sockfd = -1;
			outptr[0] = digitalRead(dioRStat, dioRMsg,recvDioR, ip, dev, subdev, channel);
		      
		      break;

		      
  
  case Ending:

		      /* frees all allocated pointers */
		      /* comedi_open */
		      scicos_free(openMsg);
		      openMsg = NULL;

		      scicos_free(openStat);
		      openStat = NULL;

		      scicos_free(recvOpen);
		      recvOpen = NULL;


                      /* comedi_dio_read */
		      scicos_free(dioRMsg);
		      dioRMsg = NULL;

		      scicos_free(dioRStat);
		      dioRStat = NULL;

		      scicos_free(recvDioR);
		      recvDioR = NULL;
		      
		      
		      /* comedi_get_subdevice_type */
		      scicos_free(typeMsg);
		      typeMsg = NULL;

		      scicos_free(typeStat);
		      typeStat = NULL;

		      scicos_free(recvType);
		      recvType = NULL;
		      
		      break;	  
		              
  } // close the switch
   
} // close the function

