/* 
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia 
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de M. Silva
 * Projeto de Graduacao - Adequacao do Software 
 * de Simulacao Scicos para a Aquisicao de Dados
 * e o Controle em Tempo Real
 * -------------------------------------------------------------------------------------------
 * Comedi acquisition server: connection module. Opens socket, binds with SERVER_PORT. 
 * 
 * Revision 1.1  2009/08/03 01:58:47  elaine
 * Initial revision
 *
 * Revision 1.2  2009/12/05 10:47:52  elaine
 * Minor enhancements
 * 
 * Revision 1.3  2010/01/04 23:40:00  elaine
 * Simplified the bind call
 * -------------------------------------------------------------------------------------------
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stddef.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <comedilib.h>   // provides comedi functions
#include "includes/comedisrv.h"   // provides comedi modified functions and constants


int getLocalSocket(int port){

		
	/* variables */
	int stat, sockfd = -1;
	int yes = 1;
	struct sockaddr_in myAddr;

        memset(&myAddr,0,sizeof(myAddr)); 
	myAddr.sin_family = AF_INET;
	myAddr.sin_port = htons(port);
	myAddr.sin_addr.s_addr = INADDR_ANY;
	

	if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0){
		perror("acqd: socket");
		return -1;
	}

	// prevents "address in use" error message. Allows the reuse of addresses 
	stat = setsockopt(sockfd,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int));
	if (stat < 0){
		perror("setsockopt(SO_REUSEADDR) failed");
		return -1;
	}


	// bind gets the unique socket name
	if(bind(sockfd,(struct sockaddr *) &myAddr, sizeof(myAddr)) < 0){
		close(sockfd); // on failure closes bad socket and tries to open a new one
		perror("acqd: bind");
		return -1;
	}
	

	return(sockfd);


}
