/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de M. Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para a Aquisicao de Dados
 * e o Controle em Tempo Real
 * ------------------------------------------------------------------------------------------
 *  Comedi acquisition client: Assembles messages (acquisition requests)  to send to server
 *
 *  Revision 1.1  2009/06/209 00:00:00  elaine
 *  Initial revision
 *
 *  Revision 1.2  2009/12/05  00:00:00  elaine
 *  Minor enhancements
 *
 *  -----------------------------------------------------------------------------------------
 *
 */


/* required header files */

#include "../includes/stack-c.h" // Scilab stack functions
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "../includes/comedisrv.h"  // provides comedi modified functions and constants


void remoteModprobe(char *acq,char *opt, char *modprobeMsg){

	// clears modprobeMsg buffer
	bzero(modprobeMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_modprobe/-";

	// starts concatenations to create acquisition request
	strcat(modprobeMsg,msg);
	strcat(modprobeMsg,opt);
	strcat(modprobeMsg,"/");
	strcat(modprobeMsg, acq);
	strcat(modprobeMsg, "/");

}


void remoteComediConfig(char *dev,char *acq,char *io,char *irq, char *opt, char *configMsg){

	// clears configMsg buffer
	bzero(configMsg, BUFFERSIZE * sizeof(char));

	// base command
        char *msg = "comedi_config/-";

	// starts concatenation 
	strcat(configMsg,msg);
	strcat(configMsg,opt);
	strcat(configMsg, "/");
	strcat(configMsg,dev);
	strcat(configMsg, "/");
	strcat(configMsg,acq);
	strcat(configMsg,"/");

	//if io and IRQ are not empty add them to the message
	if (strcmp(io," "))
		strcat(configMsg,io);
	if(strcmp(irq, " ")){
		strcat(configMsg,"/");
		strcat(configMsg,irq);

	}

}


void remoteComediOpen(char *dev,char *openMsg){

	// clears openMsg buffer
	bzero(openMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_open/";

	// starts concatenations
	strcat(openMsg,msg);
	strcat(openMsg, dev);
	strcat(openMsg, "/");
	
	

}



void remoteComediGetMaxdata(char *dev, char *subdev, char *channel, char *maxDataMsg){

	// clears maxDataMsg buffer
	bzero(maxDataMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_get_maxdata/";

	// starts concatenations
	strcat(maxDataMsg,msg);
	strcat(maxDataMsg,dev);
	strcat(maxDataMsg,"/");
	strcat(maxDataMsg,subdev);
	strcat(maxDataMsg,"/");
	strcat(maxDataMsg,channel);
	
	

}

void remoteComediGetRange(char *dev, char *subdev, char *channel, char *range, char *rangeMsg){

	// clears rangeMsg buffer
	bzero(rangeMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_get_range/";

	// starts concatenations
	strcat(rangeMsg,msg);
	strcat(rangeMsg,dev);
	strcat(rangeMsg,"/");
	strcat(rangeMsg,subdev);
	strcat(rangeMsg,"/");
	strcat(rangeMsg,channel);
	strcat(rangeMsg,"/");
	strcat(rangeMsg,range);
	
}

void remoteComediToPhys(char *data, char *maxdata, char *toPhysMsg,char *min, char *max, char *unit){

	// clears toPhysMsg buffer
	bzero(toPhysMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_to_phys/";

	// starts concatenations
	strcat(toPhysMsg,msg);
	strcat(toPhysMsg,data);
	strcat(toPhysMsg,"/");
	strcat(toPhysMsg,maxdata);
	strcat(toPhysMsg,"/");
	strcat(toPhysMsg,min);
	strcat(toPhysMsg,"/");
	strcat(toPhysMsg,max);
	strcat(toPhysMsg,"/");
	strcat(toPhysMsg,unit);
		
}

void remoteComediFromPhys(char *data, char *maxdata, char *fromPhysMsg){

	// clears fromPhysMsg buffer
	bzero(fromPhysMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_from_phys/";

	// starts concatenations
	strcat(fromPhysMsg,msg);
	strcat(fromPhysMsg,data);
	strcat(fromPhysMsg,"/");
	strcat(fromPhysMsg,maxdata);
		
}

void remoteComediDataRead(char *dev, char *subdev, char *channel, char *range, char *aref,char *dataReadMsg){

	// clears dataReadMsg buffer
	bzero(dataReadMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_data_read/";

	// starts concatenations
	strcat(dataReadMsg,msg);
	strcat(dataReadMsg,dev);
	strcat(dataReadMsg,"/");
	strcat(dataReadMsg,subdev);
	strcat(dataReadMsg,"/");
	strcat(dataReadMsg,channel);
	strcat(dataReadMsg,"/");
	strcat(dataReadMsg,range);
	strcat(dataReadMsg,"/");
	strcat(dataReadMsg,aref);

}

void remoteComediDioRead(char *dev, char *subdev, char *channel, char *dioRMsg){

	// clears dioRMsg buffer
	bzero(dioRMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_dio_read/";

	// starts concatenations
	strcat(dioRMsg,msg);
	strcat(dioRMsg,dev);
	strcat(dioRMsg,"/");
	strcat(dioRMsg,subdev);
	strcat(dioRMsg,"/");
	strcat(dioRMsg,channel);


}


void remoteComediDioWrite(char *dev, char *subdev, char *channel, char *data, char *dioWMsg){

	// clears dioWMsg buffer
	bzero(dioWMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_dio_write/";

	// starts concatenations
	strcat(dioWMsg,msg);
	strcat(dioWMsg,dev);
	strcat(dioWMsg,"/");
	strcat(dioWMsg,subdev);
	strcat(dioWMsg,"/");
	strcat(dioWMsg,channel);
	strcat(dioWMsg,"/");
	strcat(dioWMsg,data);


}



void remoteComediDataWrite(char *dev, char *subdev, char *channel, char *range, char *aref, char *data, char *dataWriteMsg){

	// clears dataWriteMsg buffer
	bzero(dataWriteMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_data_write/";

	// starts concatenations
	strcat(dataWriteMsg,msg);
	strcat(dataWriteMsg,dev);
	strcat(dataWriteMsg,"/");
	strcat(dataWriteMsg,subdev);
	strcat(dataWriteMsg,"/");
	strcat(dataWriteMsg,channel);
	strcat(dataWriteMsg,"/");
	strcat(dataWriteMsg,range);
	strcat(dataWriteMsg,"/");
	strcat(dataWriteMsg,aref);
	strcat(dataWriteMsg,"/");
	strcat(dataWriteMsg,data);

}


void remoteGetVersion(char *dev, char *getVerMsg){

	// clears getVerMsg buffer
	bzero(getVerMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_get_version_code/";

	// starts concatenations
	strcat(getVerMsg,msg);
	strcat(getVerMsg,dev);
	

}

void remoteGetDriver(char *dev, char *getDriverMsg){

	// clears getDriverMsg buffer
	bzero(getDriverMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_get_driver_name/";

	// starts concatenations
	strcat(getDriverMsg,msg);
	strcat(getDriverMsg,dev);
	

}


void remoteGetNSubdevices(char *dev,  char *NSubdevMsg){

        // clears NSubdevMsg buffer
	bzero(NSubdevMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_get_n_subdevices/";

	// starts concatenations
	strcat(NSubdevMsg,msg);
	strcat(NSubdevMsg,dev);
	

}

void remoteGetSubdeviceType(char *dev, char *subdev, char *getSubdevTypeMsg){

	// clears getSubdevTypeMsg buffer
	bzero(getSubdevTypeMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_get_subdevice_type/";

	// starts concatenations
	strcat(getSubdevTypeMsg,msg);
	strcat(getSubdevTypeMsg,dev);
	strcat(getSubdevTypeMsg,"/");
	strcat(getSubdevTypeMsg,subdev);

}


void remoteFindRange(char *dev, char *findRangeMsg){

	// clears findRangeMsg buffer
	bzero(findRangeMsg, BUFFERSIZE * sizeof(char));

	// base command
	char *msg = "comedi_find_range/";

	// starts concatenations
	strcat(findRangeMsg,msg);
	strcat(findRangeMsg,dev);
	

}
