/* 
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia 
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de M. Silva
 * Projeto de Graduacao - Adequacao do Software 
 * de Simulacao Scicos para a Aquisicao de Dados
 * e o Controle em Tempo Real
 * -------------------------------------------------------------------------------------------
 * Comedi acquisition server: board selection module. Selects board driver and model from
 * config file
 *
 * Revision 1.1  2009/08/03 02:00:30  elaine
 * Initial revision
 *
 * Revision 1.2  2009/12/05 11:32:00  elaine
 * Minor enhancements
 *
 * -------------------------------------------------------------------------------------------
 *
 */

/* required header files */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stddef.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <comedilib.h>   // provides comedi fucntions
#include "includes/comedisrv.h"   // provides comedi modified functions and constants


int getDriverName(char *boardName, int maxLineSize, char *option, char *driverModelName){

	
	/* opens comedi-driver.conf - config file */
	FILE *driverFile = fopen("comedi-driver.conf","r");
	if (!driverFile){
		fprintf(stderr, "%s: comedi config file failure: %s\n",strerror(errno),"comedi-driver.conf");
		return -1;
	}

	/* variables */
	char lineRead[maxLineSize], auxLineRead[maxLineSize], *nameFound;
	int found = 0;
	int tam = 0;
	const char *delimiter = ":";
	const char *driver = "driver";
	const char *model = "model";

	char *driverModelNamePtr = 0;


	while (found == 0 && fgets(lineRead,maxLineSize,driverFile) != NULL){
		
	
		/* clears line buffers */
		bzero(auxLineRead,sizeof(char) * maxLineSize);
		bzero(driverModelName,sizeof(char) *maxLineSize);


		strcpy(auxLineRead,lineRead);
		nameFound = strtok(auxLineRead,delimiter); /* catches first field */
		if(strcmp(boardName,nameFound) == 0 ){     /* if the board name is equal to the first field of the file */
			if(strcmp(option,driver) == 0){    /* and if the option is "driver" */
				driverModelNamePtr = strtok(NULL,delimiter); /* catches the module name to be loaded to kernel */
				strcpy(driverModelName, driverModelNamePtr);
				tam = strlen(driverModelName);
				if(driverModelName[tam-1] == '\n'){
					driverModelName[tam-1] = '\0';
				}
				found = 1;

			}

			if(strcmp(option,model) == 0){ /* if the option is "model" */

				if (( driverModelNamePtr = strrchr(lineRead,':')) != 0){ /* catches model name */

					++driverModelNamePtr; /* skips ":" */
					strcpy(driverModelName, driverModelNamePtr);
					tam = strlen(driverModelName);
					found = 1;
					if(driverModelName[tam-1] == '\n'){
						driverModelName[tam-1] = '\0';
					}
				}

			}

		}

	}
	fclose(driverFile);
	return 0;

}
