/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de M. Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para a Aquisicao de Dados
 * e o Controle em Tempo Real
 * ------------------------------------------------------------------------------------------
 *  Comedi acquisition client: Acquisition functions for Scicos blocks
 *
 *  Revision 1.1  2010/01/05 22:29:00  elaine
 *  Initial revision
 *  -----------------------------------------------------------------------------------------
 *
 */


/* required header files */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <arpa/inet.h>
#include "../includes/scicos_block4.h"  /* Specific for scicos block developement */
//#include <comedilib.h>     /* Specific for Comedi Interface */
#include "../includes/machine.h"
#include "../includes/comedisrv.h"  // provides comedi modified functions and constants

typedef unsigned int lsampl_t;



void verifySubdeviceType(char *typeStat, char *typeMsg,char *recvType, char *ip, char *dev, char *subdev, int *typesAllowed, int typesCount){

	int subdevType;
	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
	int counter, typesNotMatched = 0;
          
	/* establishes a connection with the server. If not possible, exits */
	if ((sockfd = connectToServer(ip)) > 0){

		/* assembles message to send to server */
		remoteGetSubdeviceType(dev,subdev,typeMsg);

		/* sends message to server. If the number of bytes is more than 0
		 * receiver message from server. If not, exits */
		if ((sentBytes = clientSender(sockfd, typeMsg)) >= 0){
			
			/* receives msg from server. If the number of bytes read is more
			 * than 0 , checks the return code from comedi_get_subdevice_type executed 
			 * remotely and the returned msg. If not, exits. */
			if ((receiveStatus = clientReceiver(sockfd,typeStat)) > 0){
				
				/* checks the status of the comedi_get_subdevice_type function. If ok
				 * converts returned values. If not, the program aborts */
				rc = checkCmdStatus(typeStat, recvType);
				if (rc == 0){
					subdevType = atoi (recvType);
					for (counter = 0; counter < typesCount; ++counter){
						if (subdevType != typesAllowed[counter]) {
						         ++typesNotMatched;
						}
					}
					if (typesNotMatched == typesCount){
						Scierror(888,"Incorrect subdevice type. Please check BOARD manual\n");
						set_block_error(-1);
						end_scicos_sim();
					}

				}
				else{
					Scierror(888,"Error in comedi_get_subdevice_type.\n");
					set_block_error(-1);
					end_scicos_sim();
				}
			}
			else{
				Scierror(888,"It was not possible to receive msgs from the server\n");
				set_block_error(-1);
				end_scicos_sim();
			}

		}
		else{
			Scierror(888,"It was not possible to send message: '%s' to server\n", typeMsg);
			set_block_error(-1);
			end_scicos_sim();
		}

		close(sockfd);
	}
	else{
		Scierror(888,"It was not possible to open a connection with %s.\n",ip);
		set_block_error(-1);
		end_scicos_sim();
	}
		 
}


void openComediDevice(char *openStat, char *openMsg,char *recvOpen, char *ip, char *dev){

	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;

	/* establishes a connection with the server. If not possible, exits */
	if ((sockfd = connectToServer(ip)) > 0){
		/* assembles message to send to server */
		remoteComediOpen(dev,openMsg);
		
		/* sends message to server. If the number of bytes is more than 0 receives 
		 * message from server. If not, exits */
		if((sentBytes = clientSender(sockfd,openMsg)) >=  0){
			/* receives msg from server. If the number of bytes read is more than 
			 * 0 checks the return code from comedi_open executed remotely and the 
			 * msg returned. If not, exits. */
			if ((receiveStatus = clientReceiver(sockfd,openStat)) > 0){
				/* checks the status of the comedi_open function. If status ok 
				 * the program continues. If not ok program aborts */
				rc = checkCmdStatus(openStat, recvOpen);
				if (rc != 0){
					Scierror(888,"Error on comedi_open.\n");
					set_block_error(-1);
					end_scicos_sim();
				}
			}
			else{
				Scierror(888,"It was not possible to receive message from server\n");
				set_block_error(-1);
				end_scicos_sim();
			}
		}
		else{
			Scierror(888,"It was not possible to send message: '%s' to server\n",openMsg);
			set_block_error(-1);
			end_scicos_sim();
		}
		close(sockfd);
	}
	else{
		Scierror(888,"It was not possible to open a connection with %s.\n",ip);
		set_block_error(-1);
		end_scicos_sim();
	}

}


int digitalRead(char *dioRStat, char *dioRMsg, char *recvDioR, char *ip, char *dev, char *subdev, char *channel){

	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
	lsampl_t data = 0;

	/* establishes a connection with the server. If not possible, exits */
	if ((sockfd = connectToServer(ip)) > 0){
		/* assembles message to send to server */
		remoteComediDioRead(dev, subdev, channel, dioRMsg);
		/* sends message to server. If the number of bytes is more than 0
		 * receiver message from server. If not, exits */
		if ((sentBytes = clientSender(sockfd, dioRMsg)) >= 0){
			/* receives msg from server. If the number of bytes read is more
			 * than 0 , checks the return code from comedi_open executed 
			 * remotely and the returned msg. If not, exits. */
			if ((receiveStatus = clientReceiver(sockfd, dioRStat)) > 0){
				/* checks the status of the comedi_dio_read function. If ok
				 * converts returned values. If not, the program aborts */
				rc = checkCmdStatus(dioRStat, recvDioR);
				if (rc == 0){
					data = atoi (recvDioR);
				}
				else{
					Scierror(888,"Error in comedi_dio_read.\n");
					set_block_error(-1);
					end_scicos_sim();
				}
			}
			else{
				Scierror(888,"It was not possible to receive msgs from the server\n");
				set_block_error(-1);
				end_scicos_sim();
			}
		}
		else{
			Scierror(888,"It was not possible to send message: '%s' to server\n", dioRMsg);
			set_block_error(-1);
			end_scicos_sim();
		}
		close(sockfd);
	}

	else{
		Scierror(888,"It was not possible to open a connection with %s.\n",ip);
		set_block_error(-1);
		end_scicos_sim();

		}

	return data;


}



void getMaxData(char *maxDataStat, char *maxDataMsg,char *recvMaxData, char *ip, char *dev, char *subdev, char *channel){

	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
	
	/* establishes a connection with the server. If not possible, exits */
	if ((sockfd = connectToServer(ip)) > 0){

		/* assembles message to send to server */
		remoteComediGetMaxdata(dev,subdev, channel, maxDataMsg);
		
		/* sends message to server. If the number of bytes is more than 0 receives 
		 * message from server. If not, exits */
		if((sentBytes = clientSender(sockfd,maxDataMsg)) >=  0){
			/* receives msg from server. If the number of bytes read is more than 
			 * 0 checks the return code from comedi_get_maxdata executed remotely and the 
			 * msg returned. If not, exits. */
			if ((receiveStatus = clientReceiver(sockfd,maxDataStat)) > 0){
				/* checks the status of the comedi_get_maxdata function. If status ok 
				 * the program continues. If not ok program aborts */
				rc = checkCmdStatus(maxDataStat, recvMaxData);
				if (rc < 0){
					Scierror(888,"Error in comedi_get_maxdata.\n");
					set_block_error(-1);
					end_scicos_sim();
				}
			}
			else{
				Scierror(888,"It was not possible to receive msgs from the server\n");
				set_block_error(-1);
				end_scicos_sim();
			}
		}
		else{
			Scierror(888,"It was not possible to send message: '%s' to server\n", maxDataMsg);
			set_block_error(-1);
			end_scicos_sim();
		}
		close(sockfd);
	}
	else{
		Scierror(888,"It was not possible to open a connection with %s.\n",ip);
		set_block_error(-1);
		end_scicos_sim();
	}

}


			
void getRange(char *rangeStat, char *rangeMsg,char *recvRange, char *ip, char *dev, char *subdev, char *channel, char *range, char *minRange, char *maxRange, char *unitRange){


	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
	const char *delimiter = "/";
	char *token;
	int tokenCount = 0;
	static char *rangeStruct[3];

	/* establishes a connection with the server. If not possible, exits */
	if ((sockfd = connectToServer(ip)) > 0){
		/* assembles message to send to server */
		remoteComediGetRange(dev,subdev, channel, range, rangeMsg);
		/* sends message to server. If the number of bytes is more than 0 receives 
		 * message from server. If not, exits */
		if((sentBytes = clientSender(sockfd,rangeMsg)) >=  0){
			/* receives msg from server. If the number of bytes read is more than 
			 * 0 checks the return code from comedi_get_range executed remotely and the 
			 * msg returned. If not, exits. */
			if ((receiveStatus = clientReceiver(sockfd,rangeStat)) > 0){
				/* checks the status of the comedi_get_range function. If status ok 
				 * the program continues. If not ok program aborts */
				rc = checkCmdStatus(rangeStat, recvRange);
				if (rc >= 0){
					for (token = strtok(recvRange, delimiter); (token != NULL) && (tokenCount < 3) ; token = strtok(NULL, delimiter)) {
						rangeStruct[tokenCount] = token;
						tokenCount++;
					}
					strcpy(rangeStruct[0], minRange);
					strcpy(rangeStruct[1], maxRange);
					strcpy(rangeStruct[2], unitRange);

				}
				else{
					Scierror(888,"Error in comedi_get_range.\n");
					set_block_error(-1);
					end_scicos_sim();
				}
			}
			else{
				Scierror(888,"It was not possible to receive msgs from the server\n");
				set_block_error(-1);
				end_scicos_sim();
			}
		}
		else{
			Scierror(888,"It was not possible to send message: '%s' to server\n", rangeMsg);
			set_block_error(-1);
			end_scicos_sim();
		}
		
		close(sockfd);

	}
	else{
		Scierror(888,"It was not possible to open a connection with %s.\n",ip);
		set_block_error(-1);
		end_scicos_sim();
	}
				
}

void analogRead(char *dataReadStat, char *dataReadMsg,char *recvDataRead, char *ip, char *dev, char *subdev, char *channel, char *range, char *aref){

	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
	/* establishes a connection with the server. If not possible, exits */
	if ((sockfd = connectToServer(ip)) > 0){
		/* assembles message to send to server */
		remoteComediDataRead(dev,subdev,channel,range,aref,dataReadMsg);
		/* sends message to server. If the number of bytes is more than 0
		 * receiver message from server. If not, exits */
		if ((sentBytes = clientSender(sockfd, dataReadMsg)) >= 0){
			/* receives msg from server. If the number of bytes read is more
			 * than 0 , checks the return code from comedi_open executed 
			 * remotely and the returned msg. If not, exits. */
			if ((receiveStatus = clientReceiver(sockfd,dataReadStat)) > 0){
				/* checks the status of the comedi_data_read function. If ok
				 * converts returned values. If not, the program aborts */
				rc = checkCmdStatus(dataReadStat, recvDataRead);
				if (rc < 0){
					Scierror(888,"Error in comedi_data_read.\n");
					set_block_error(-1);
					end_scicos_sim();
				}
			}
			else{
				Scierror(888,"It was not possible to receive msgs from the server\n");
				set_block_error(-1);
				end_scicos_sim();
			}
		}
		else{
			Scierror(888,"It was not possible to send message: '%s' to server\n", dataReadMsg);
			set_block_error(-1);
			end_scicos_sim();
		}
		close(sockfd);
	}
	else{
		Scierror(888,"It was not possible to open a connection with %s.\n",ip);
		set_block_error(-1);
		end_scicos_sim();
	}
		      
}


/* converts samples to physical values */
double comediToPhys(char *recvMaxData, char *recvDataRead, char *toPhysStat, char *toPhysMsg, char *recvToPhys, char *min, char *max, char *unit, char *ip){

	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
	double dataPhys = 0.000;

	/* establishes a connection with the server. If not possible, exits */
		      if ((sockfd = connectToServer(ip)) > 0){
			      /* assembles message to send to server */
			      remoteComediToPhys(recvDataRead,recvMaxData, toPhysMsg, min, max, unit);

			      /* sends message to server. If the number of bytes is more than 0
			       * receiver message from server. If not, exits */
			      if ((sentBytes = clientSender(sockfd, toPhysMsg)) >= 0){

				      /* receives msg from server. If the number of bytes read is more
				       * than 0 , checks the return code from comedi_open executed 
				       * remotely and the returned msg. If not, exits. */
				      if ((receiveStatus = clientReceiver(sockfd,toPhysStat)) > 0){

					      /* checks the status of the comedi_to_phys function. If ok
					       * converts returned values. If not, the program aborts */
					     

					      rc = checkCmdStatus(toPhysStat, recvToPhys);
					      if (rc >= 0){
						    
						      dataPhys = atof (recvToPhys);
					         }
					      else{
						       Scierror(888,"Error in comedi_to_phys.\n");
				               set_block_error(-1);
				               end_scicos_sim();
						      }

					      }
				      else{

					       Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
				               end_scicos_sim();
					      
					      
				      }

				      }

			      else{
				       Scierror(888,"It was not possible to send message: '%s' to server\n", toPhysMsg);
				               set_block_error(-1);
				               end_scicos_sim();
				      
				  				  
			      }

			      close(sockfd);
			   //   printf("depois de fechar socket:%d\n",sockfd);

		      }
		      
		      
		      else{

			     Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
				end_scicos_sim();
				}
				               

return dataPhys;			     		      
}


/* converts physical values to samples */
void comediFromPhys(double dataPhys, char *recvMaxData, char *fromPhysMsg, char * fromPhysStat, char *recvFromPhys, char *ip){
int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
char dataPhysChar[BUFFERSIZE];

/* establishes a connection with the server. If not possible, exits */
			if ((sockfd = connectToServer(ip)) > 0){
		

				sprintf(dataPhysChar, "%f", dataPhys);
				
				remoteComediFromPhys(dataPhysChar,recvMaxData, fromPhysMsg);
				
				

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits 
				 */
				if((sentBytes = clientSender(sockfd,fromPhysMsg)) >=  0){
					
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_from_phys executed remotely and the 
					 * msg returned. If not, exits. 
					 */
					if ((receiveStatus = clientReceiver(sockfd,fromPhysStat)) > 0){

						/* checks the status of the comedi_from_phys function. If status ok 
						 * the program continues. If not ok program aborts */

						
						rc = checkCmdStatus(fromPhysStat, recvFromPhys);
				                  if (rc != 0) {  
				                  Scierror(888,"Error in comedi_from_phys.\n");
				                        set_block_error(-1);
				                        end_scicos_sim();
						      

					      }
					      }
				      else{
					      Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
				               end_scicos_sim();
				      }

				      }

			      else{
				      Scierror(888,"It was not possible to send message: '%s' to server\n", fromPhysMsg);
				               set_block_error(-1);
				               end_scicos_sim();
			      }

			      close(sockfd);

							

			}
			else{
				 Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
				end_scicos_sim();
			}
			
			



}


void analogWrite(char *dataWriteStat, char *dataWriteMsg,char *recvDataWrite, char * recvFromPhys, char *ip, char *dev, char *subdev, char *channel, char *range, char *aref){

int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;


       		/* establishes a connection with the server. If not possible, exits */
			if (((sockfd = connectToServer(ip)) > 0)) {
		

				/* assembles message to send to server */
				remoteComediDataWrite(dev,subdev, channel,range, aref,recvFromPhys, dataWriteMsg);

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits 
				 */
				if((sentBytes = clientSender(sockfd,dataWriteMsg)) >=  0){
					
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_data_write executed remotely and the 
					 * msg returned. If not, exits. 
					 */
					if ((receiveStatus = clientReceiver(sockfd,dataWriteStat)) > 0){

						/* checks the status of the comedi_data_write function. If status ok 
						 * the program continues. If not ok program aborts 
						 */

						
						rc = checkCmdStatus(dataWriteStat, recvDataWrite);
				                  if (rc != 0){
				                  
						      Scierror(888,"Error in comedi_data_write.\n");
				               set_block_error(-1);
				               end_scicos_sim();
						      }
						      }

				      else{
					      Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
				               end_scicos_sim();
				      }

				     }

			      else{
				       Scierror(888,"It was not possible to send message: '%s' to server\n", dataWriteMsg);
				               set_block_error(-1);
				               end_scicos_sim();
			      }

			      close(sockfd);

							

			}
			else{
				 Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
				end_scicos_sim();
			}
			}

			

















