/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia 
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de Mattos Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para Aquisicao de Dados
 * e o Controle em Tempo Real
 * ------------------------------------------------------------------------------------------
 * Comedi acquisition client: Gets the subdevice type. 
 * 
 * Revision 1.1  2009/12/28  13:47:00 elaine
 * Initial revision 
 *
 * ------------------------------------------------------------------------------------------
 *
 */


/* required header files */
#include "../includes/stack-c.h" // provides Scilab stack functions
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "../includes/comedisrv.h"  // provides comedi modified functions and constants


void getSubdevType(char *ip,char *dev, char *subdev, char **recvType, int *m4){

	int sentBytes, receiveStatus;
	int sockfd = -1, rc = -1;
	char *getTypeStat, *getTypeMsg;
	char * openMsg, *openStat, *recvOpen;
	int openFlag = 0;
	
	if ((openMsg  = (char *)malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				printf("Not enough space to allocate openMsg\n");
				exit(1);
			}

			if ((openStat = (char *)malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				printf("Not enough space to allocate openStat\n");
				exit(1);
			}

			if ((recvOpen = (char *) malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				printf("Not enough space to allocate recvOpen\n");
				exit(1);
			}

	if((getTypeStat = (char *) malloc ((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
		printf("getBoardType: Not enough space to allocate getTypeStat\n");
		exit(1);
		}

	if((getTypeMsg = (char *) malloc ((unsigned) (BUFFERSIZE * sizeof(char)))) == NULL){
		printf("getBoardType: Not enough space to allocate getTypeMsg\n");
		exit(1);
		}

	
	if((*recvType = (char *) malloc((unsigned) (BUFFERSIZE * sizeof(char)))) == NULL){
		printf("getBoardType: Not enough space to allocate recvType\n");
		exit(1);
	}


                           /*
			 * opens device 
			 */
			 
			/* establishes a connection with the server. If not possible, exits */
			if ((sockfd = connectToServer(ip)) > 0){
		

				/* assembles message to send to server */
				remoteComediOpen(dev,openMsg);

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits */
				if((sentBytes = clientSender(sockfd,openMsg)) >=  0){
					
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_open executed remotely and the 
					 * msg returned. If not, exits. */
					if ((receiveStatus = clientReceiver(sockfd,openStat)) > 0){

						/* checks the status of the comedi_open function. If status ok 
						 * the program continues. If not ok program aborts */

						
						rc = checkCmdStatus(openStat, recvOpen);
						if (rc != 0){
							bzero(*recvType, BUFFERSIZE * sizeof(char));
                                            	                 strcat(*recvType, "It was not possible to open the device!");
                                            	                 strcat(*recvType, recvOpen);		                
                                            		                 *m4 = strlen(*recvType);
                                            		                 return(0);
							}
						else{
						//printf("comedi_open message: '%s'\n",recvOpen);
						openFlag = 1;
						}

					}

					else{
						//printf("It was not possible to receive message from server\n");
						bzero(*recvType, BUFFERSIZE * sizeof(char));
                                            	        *recvType = "It was not possible to receive message from server";
                                  		                 *m4 = strlen(*recvType);
                                  		                 return(0);
						}
					}
				else{
						//printf("It was not possible to send message: '%s' to server\n",openMsg);
						bzero(*recvType, BUFFERSIZE * sizeof(char));
                                            	        strcat(*recvType,"It was not possible to send message: ");
                                  		                 strcat(*recvType, openMsg);
                                  		                 *m4 = strlen(*recvType);
                                  		                 return(0);
					}
				close(sockfd);

							

			}
			else{
                                  // printf("Open: It was not possible to open a connection with %s\n",ip);
                                 		//exit(1);
                                 		bzero(*recvType, BUFFERSIZE * sizeof(char));
		                 strcat(*recvType, "Open: It was not possible to open a connection with ");
		                 strcat(*recvType, ip);		                
		                 *m4 = strlen(*recvType);
		                 return(0);
			}
			

	/* tries to connect to server. On success sends acquisition request */
	if (((sockfd = connectToServer(ip)) > 0) && (openFlag = 1)) {
		remoteGetSubdeviceType(dev, subdev, getTypeMsg); // assembles message to send 
		if ((sentBytes = clientSender(sockfd,getTypeMsg)) >= 0){	// sends message to socket, returns number of bytes sent

			if ((receiveStatus = clientReceiver(sockfd,getTypeStat)) > 0) {  // receives message from server,returns number of bytes read and closes socket 

				
				rc = checkCmdStatus(getTypeStat,*recvType); // strips message from HTML protocol, strips into return code and message

				if (rc == 0){
					*m4 = strlen(*recvType);
					}
				if (rc == -1){
				*m4 = strlen(*recvType);
				return(0);

					}
				} // end of clientReceiver if
				else{
				//printf("It was not possible to receive message from server\n");
						bzero(*recvType, BUFFERSIZE * sizeof(char));
                                            	        *recvType = "It was not possible to receive message from server";
                                  		                 *m4 = strlen(*recvType);
                                  		                 return(0);
                                  		                 }
			} // end of clientSender if
			else{
			//printf("It was not possible to send message: '%s' to server\n",openMsg);
						bzero(*recvType, BUFFERSIZE * sizeof(char));
                                            	        strcat(*recvType,"It was not possible to send message: ");
                                  		                 strcat(*recvType, openMsg);
                                  		                 *m4 = strlen(*recvType);
                                  		                 return(0);
                                  		                 }
                                  		                 
	}//end of the connectToServer if 
	else{
                                 //  printf("Open: It was not possible to open a connection with %s\n",ip);
                                 		//exit(1);
                                 		bzero(*recvType, BUFFERSIZE * sizeof(char));
		                 strcat(*recvType, "Open: It was not possible to open a connection with ");
		                 strcat(*recvType, ip);		                
		                 *m4 = strlen(*recvType);
		                 return(0);
			}


	//printf("Antes dos frees\n");
 	free(getTypeStat);
	free(getTypeMsg);
	getTypeStat = NULL;
	getTypeMsg = NULL;
	//printf("depois dos frees\n");


	
}

