/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de M. Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para a Aquisicao de Dados
 * e o Controle em Tempo Real
 * ------------------------------------------------------------------------------------------
 * Comedi acquisition client: Sends, receives and checks messages
 *
 *  Revision 1.1  2009/06/209 00:00:00  elaine
 *  Initial revision
 *
 *  Revision 1.2  2009/12/05  19:18:00  elaine
 *  Minor enhancements
 *
 *  -----------------------------------------------------------------------------------------
 *
 */


/* required header files */

#include "/home/elaine/scilab-4.1.2/routines/stack-c.h" // Scilab stack functions
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "../includes/comedisrv.h"  // provides comedi modified functions and constants

int clientSender(int sockfd, char *sendMsg){

	int sentBytes = 0;

	// prepares messages to be sent to server - wraps in HTTP protocol
	// TODO - adicionar /n/n ao fim da msg a ser enviada. Motivo:ficar compliant com protocolo. 
	// No servidor o /n/n sera tratado como fim de transmissao 
	char *httpSendHeader = "GET /";
	char *sendBuffer;
	if((sendBuffer	=  (char *) malloc((unsigned) (BUFFERSIZE * sizeof(char)))) == NULL){
		printf("clientSender: Not enough space to allocate sendBuffer\n");
		exit(1);
	}

	// clears sendBuffer buffer
	bzero(sendBuffer, sizeof sendBuffer);

	// copies HTTP header            
	strcpy(sendBuffer,httpSendHeader); 
		
	// starts concatenation series
	strcat(sendBuffer,sendMsg);
	strcat(sendBuffer," HTTP/1.1");

	// sends message to server    
	sentBytes = write(sockfd,sendBuffer,strlen(sendBuffer));
	if(sentBytes < 0){
		perror("client: write");
		exit(1);
		}

	free(sendBuffer);
	sendBuffer = NULL;

	//returns number of bytes sent. If < 0 there was an error. 
	return sentBytes;

}

int clientReceiver(int sockfd, char *recvMsg){

	int bytesRead = 0;
	int numBytes = 0;
	char buff[BUFFERSIZE];
	char buffAux[BUFFERSIZE];

	// clears buffers 
	bzero(recvMsg, sizeof(char) * BUFFERSIZE);
	bzero(buff, BUFFERSIZE * sizeof(char));  
	bzero(buffAux, BUFFERSIZE * sizeof(char));

	// receives server messages (max. BUFFERSIZE size)
	while(numBytes = read(sockfd, buff, BUFFERSIZE -1)!= 0) { 
		if(numBytes == -1){
			perror("read");
			return numBytes;
			}
		strcat(buffAux,buff); 
		bzero(buff,BUFFERSIZE * sizeof(char) ); 
		bytesRead = bytesRead + numBytes;

		if(numBytes < (BUFFERSIZE - 1)) // if there is no more data exits loop
			break;
		} // end while 

	strcpy(buff,buffAux);  // copies auxiliar buffer to permanent buffer

	//copies buffer w/ server messages to the Scilab stack variable
	strcpy(recvMsg,buff);
	
	return bytesRead;
}


int clientReceiverDummy(int sockfd, char *recvMsg){
  return 1;
}

int checkCmdStatus(char *recvMsg, char *editedMsg){


	int i = 0;
	int returnStatusNum = -1;
	char *returnStatusChar = ' ';
	char *parseMsgPointer;
    size_t loc = -1 ;
	char *httpHead = "HTTP\0";

	// clears the editedMsg buffer
	bzero(editedMsg, BUFFERSIZE * sizeof(char));
	
	//if message is wrapped in HTTP protocol strips it. If not just send the message to Scilab as it is
	loc = strstr(recvMsg,httpHead);
	if(loc != NULL){
	
		// finds first '\n' char on the received string 
		parseMsgPointer = strchr(recvMsg, '\n');
	
		if (parseMsgPointer != NULL){
			// advances 2 more chars to find the status number of the remote function execution. 1 = failure; 0 = success
			parseMsgPointer = parseMsgPointer + 2;
			if (parseMsgPointer == NULL){
				return -1;
				}
			else{
	  			returnStatusChar = parseMsgPointer;
	  			returnStatusNum = atoi(returnStatusChar);
				}

			// places actual message on editedMsg buffer
			parseMsgPointer = parseMsgPointer + 3;
			if (parseMsgPointer == NULL)
				return -1;
			
			while ( (*parseMsgPointer != '\n') && (i < BUFFERSIZE) && (*parseMsgPointer != '\0')){
				editedMsg[i++] = *parseMsgPointer;
				++parseMsgPointer;
				}
			editedMsg[i] = '\0';
		
	}


	}
	else{
		strcpy(editedMsg,recvMsg);
		return 0;
		
	}


	return returnStatusNum;
}

int checkCmdStatusDummy(char *recvMsg, char *editedMsg){
  return 0;
}  



